﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Timers;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace WpfApp1
{
	/// <summary>
	/// Логика взаимодействия для GameWindow.xaml
	/// </summary>
	public partial class GameWindow : Window
	{
		public Labirint labirint;
		public Vector2 sizeCell;
		public Image[,] renderMap;
		public BitmapImage[] images;
		public Timer timer;
		public string path;
		public int curStep;
		public Vector2 curChelPosition;

		public GameWindow(string filePath)
		{
			InitializeComponent();
			Closed += GameWindow_Closed;

			labirint = new Labirint(filePath);
            labirint.SaveToFile("moves.txt");
			path = string.Join("", labirint.GetPathFromStartToFinish());
			curStep = 0;
			sizeCell = new Vector2(500.0 / labirint.Width, 500.0 / labirint.Height);
			renderMap = new Image[labirint.Height, labirint.Width];
			images = new BitmapImage[4];
			images[0] = new BitmapImage(new Uri(System.IO.Path.GetDirectoryName(Assembly.GetEntryAssembly().Location) + "/Images/dirt.jpg"));
			images[1] = new BitmapImage(new Uri(System.IO.Path.GetDirectoryName(Assembly.GetEntryAssembly().Location) + "/Images/wall.png"));
			images[2] = new BitmapImage(new Uri(System.IO.Path.GetDirectoryName(Assembly.GetEntryAssembly().Location) + "/Images/chel.png"));
			images[3] = new BitmapImage(new Uri(System.IO.Path.GetDirectoryName(Assembly.GetEntryAssembly().Location) + "/Images/exit.png"));

			for (int i = 0; i < labirint.Height; i++)
			{
				for (int j = 0; j < labirint.Width; j++)
				{
					renderMap[i, j] = new Image();
					renderMap[i, j].Height = sizeCell.y;
					renderMap[i, j].Width = sizeCell.x;
					renderMap[i, j].Stretch = Stretch.Fill;
					canvas.Children.Add(renderMap[i, j]);
					Canvas.SetLeft(renderMap[i, j], j * sizeCell.x);
					Canvas.SetTop(renderMap[i, j], i * sizeCell.y);
				}
			}

			curChelPosition = labirint.startPosition;

			timer = new Timer(500);
			timer.Elapsed += OnPaint;
			timer.Start();
		}

		private void GameWindow_Closed(object sender, EventArgs e)
		{
			timer.Dispose();
		}

		public void Win()
		{
			WinWindow winWindow = new WinWindow(this, curStep);
			winWindow.Show();
		}

		public void Step()
		{
			if (curStep >= path.Length)
			{
				timer.Dispose();
				Win();
			}
			else
			{
				switch (path[curStep])
				{
					case 'R':
						curChelPosition += new Vector2(1, 0);
						break;
					case 'D':
						curChelPosition += new Vector2(0, 1);
						break;
					case 'L':
						curChelPosition += new Vector2(-1, 0);
						break;
					case 'U':
						curChelPosition += new Vector2(0, -1);
						break;
					default:
						break;
				}
				curStep++;
				StepsStat.Text = "Шаги: " + curStep;
			}
		}

		public void OnPaint(object source, ElapsedEventArgs e)
		{
			Dispatcher.Invoke(() =>
			{
				for (int i = 0; i < labirint.Height; i++)
					for (int j = 0; j < labirint.Width; j++)
						renderMap[i, j].Source = images[labirint[i, j] - 1];

				renderMap[(int)labirint.finishPosition.y, (int)labirint.finishPosition.x].Source = images[3];
				renderMap[(int)curChelPosition.y, (int)curChelPosition.x].Source = images[2];
				Step();
			});
		}
	}
}
