﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WpfApp1
{
	public class Labirint
	{
		private int[,] map;
		public Vector2 startPosition { get; private set; }
		public Vector2 finishPosition { get; private set; }
		public int Height { get { return map.GetLength(0); } }
		public int Width { get { return map.GetLength(1); } }

		public Labirint(string fileMap)
		{
			string[] lines = File.ReadAllLines(fileMap);
			int[] size = lines[0].Split(' ').Select((c) => int.Parse(c)).ToArray();
			map = new int[size[0], size[1]];
			for (int i = 0; i < size[0]; i++)
			{
				string[] cells = lines[1 + i].Split(' ');
				for (int j = 0; j < size[1]; j++)
				{
					switch (cells[j])
					{
						case ".":
							map[i, j] = 1;
							break;
						case "X":
							map[i, j] = 2;
							break;
						case "S":
							map[i, j] = 1;
							startPosition = new Vector2(j, i);
							break;
						case "E":
							map[i, j] = 1;
							finishPosition = new Vector2(j, i);
							break;
						default:
							break;
					}
				}
			}
		}

		public void SaveToFile(string fileName)
		{
			IEnumerable<char> path = GetPathFromStartToFinish();
			File.WriteAllLines(fileName, new string[2] { path.Count().ToString(), string.Join("", path) });
		}

		public IEnumerable<char> GetPathFromStartToFinish()
		{
			int[,] depthMap = GetDepthMap();
			string result = "";
			Vector2 curPosition = finishPosition;
			while (depthMap[(int)curPosition.y, (int)curPosition.x] != 0)
			{
				for (int i = 0; i < 4; i++)
				{
					Vector2 toPos = curPosition + dirs[i];
					if (toPos.y >= 0 && toPos.y < Height &&
						toPos.x >= 0 && toPos.x < Width && map[(int)toPos.y, (int)toPos.x] != 2)
					{
						if (depthMap[(int)toPos.y, (int)toPos.x] < depthMap[(int)curPosition.y, (int)curPosition.x])
						{
							switch (i)
							{
								case 0:
									result += "L";
									break;
								case 1:
									result += "U";
									break;
								case 2:
									result += "R";
									break;
								case 3:
									result += "D";
									break;
								default:
									break;
							}
							curPosition = toPos;
							break;
						}
					}
				}
			}
			return result.Reverse();
		}

		public int[,] GetDepthMap()
		{
			visit = new bool[Height, Width];
			depths = new int[Height, Width];
			for (int i = 0; i < depths.GetLength(0); i++)
			{
				for (int j = 0; j < depths.GetLength(1); j++)
				{
					depths[i, j] = int.MaxValue;
				}
			}
			depths[(int)startPosition.y, (int)startPosition.x] = 0;
			dfs(startPosition);
			return depths;
		}

		private bool[,] visit;
		private int[,] depths;
		private Vector2[] dirs = {
			new Vector2(1, 0),
			new Vector2(0, 1),
			new Vector2(-1, 0),
			new Vector2(0, -1)
		};
		private void dfs(Vector2 curPos)
		{
			visit[(int)curPos.y, (int)curPos.x] = true;
			for (int i = 0; i < 4; i++)
			{
				Vector2 toPos = curPos + dirs[i];
				if (toPos.y >= 0 && toPos.y < Height &&
					toPos.x >= 0 && toPos.x < Width && map[(int)toPos.y, (int)toPos.x] != 2)
				{
					if (visit[(int)toPos.y, (int)toPos.x])
					{
						if (depths[(int)toPos.y, (int)toPos.x] > depths[(int)curPos.y, (int)curPos.x] + 1)
							depths[(int)toPos.y, (int)toPos.x] = depths[(int)curPos.y, (int)curPos.x] + 1;
					}
					else
					{
						depths[(int)toPos.y, (int)toPos.x] = depths[(int)curPos.y, (int)curPos.x] + 1;
						dfs(toPos);
					}
				}
			}
		}

		public int this[int i, int j]
		{
			get
			{
				return map[i, j];
			}
		}
	}
}
